# -*- coding: utf-8 -*-
# MIT License
# Copyright (c) 2020 Arthur
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from typing import List

from .bot import Bot
from ..utils import represents


class UserBase:
    """Represents the base of a top.gg user."""

    def __init__(self, id: str, tag: str, avatar: str, default_avatar: str):
        """
        Creates a new Top.gg user base.

        :param id: The user their discord id
        :param tag: The user their discord tag (username#discriminator)
        :param avatar: The user their discord avatar url
        :param default_avatar: The user their default discord avatar url (calculated by discriminator)
        """
        self.id = id
        self.tag = tag
        self.avatar = f"https://cdn.discordapp.com/avatars/{id}/{avatar}.webp"
        self.default_avatar = default_avatar

    def __repr__(self):
        return represents(self)


class UserBots:
    """Represents a user with their corresponding bots."""

    def __init__(self, user: UserBase, bots: List[Bot]):
        """
        Creates a new UserBot.

        :param user: The user that owns the bots.
        :param bots: A collection of bots that are owned by the user.
        """
        self.user = user
        self.bots = bots

    def __repr__(self):
        return represents(self)



# -*- coding: utf-8 -*-
# MIT License
# Copyright (c) 2020 Arthur
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from json import loads

import aiohttp

from .endpoints import BASE_URL
from ..objects.exceptions import UnknownException, InvalidAuthorizationToken, InvalidTarget


class AsyncFetcher:
    """Handles all dblstats async events"""

    def __init__(self, auth: str):
        self.__auth = auth
        self.__header = {"Authorization": auth}

    @staticmethod
    async def process_response(res: aiohttp.ClientResponse):
        message = loads(await res.text())
        if res.status == 200 or message.get("status") == 200:
            return message
        if res.status == 400 or message.get("status") == 400:
            raise InvalidAuthorizationToken(message.get("message") or "No token has been provided!")
        elif res.status == 401 or message.get("status") == 401:
            raise InvalidAuthorizationToken(message.get("message") or "An invalid token has been provided!")
        elif res.status == 404 or message.get("status") == 404:
            raise InvalidTarget(message.get("message") or "Could not find the requested target!")
        raise UnknownException("Oops, an unknown exception got raised. Please report this to our github page "
                               "(https://github.com/Arthurdw/dblstats/issues) and provide the following content:"
                               f"Unhandled response code: {res.status or message.get('status')} "
                               f"({message.get('message')})")

    async def get(self, endpoint):
        """Sends an asynchronous get request to an endpoint"""
        async with aiohttp.ClientSession() as session:
            async with session.get(BASE_URL + endpoint, headers=self.__header) as response:
                return await self.process_response(response)



# -*- coding: utf-8 -*-
# MIT License
# Copyright (c) 2020 Arthur
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from enum import Enum
from typing import Union, List

from .bot import Bot, BotHistory
from .dblstats_auctions import Auctions
from .user import UserBots
from ..utils import represents, AsyncFetcher, endpoints
from ..utils.convertors import parse_bot_base_object, parse_bot_object, parse_user_base_object


class Sorter(Enum):
    """
    :attr votes: Sorts by most voted
    :attr monthly_votes: Sorts by most voted from this month
    :attr servers: Sorts by most servers
    :attr shards: Sorts by most shards
    """
    votes = "votes"
    monthly_votes = "monthly votes"
    servers = "servers"
    shards = "shards"


class Client:
    """
    Represents your dblstatistics.com client.

    This interacts with the api, so it allows you to fetch data.
    """

    def __init__(self, token: str):
        self.__fetcher = AsyncFetcher(token)
        self.auctions = Auctions(self.__fetcher)

    def __repr__(self):
        return represents(self)

    async def get_bot(self, id: Union[int, str]) -> Bot:
        """
        Fetch a bot from the dblstatistics website.

        :param id: The id of the bot that must be fetched.
        """
        bot = await self.__fetcher.get(endpoints.GET_BOT_URL.format(id=id))
        return parse_bot_object(bot)

    async def get_bot_history(self, id: Union[int, str]) -> BotHistory:
        """
        Fetch a bot its history from the dblstatistics website.

        :param id: The id of the bot that must be fetched.
        """
        history = await self.__fetcher.get(endpoints.GET_BOT_HISTORY_URL.format(id=id))
        return BotHistory(history.get("id"), list(map(parse_bot_base_object, history.get("data"))))

    async def get_top_bots(self, sort_by: Sorter = Sorter.votes) -> List[Bot]:
        """
        Returns a collection of sorted dblstats Bot objects.

        :param sort_by: The way the bots should get sorted, default is by votes.
        """
        if not isinstance(sort_by, Sorter):
            raise TypeError(f"Can not use type `{type(sort_by)}` to sort bots. Please use the `dblstats.Sorter` enum!")

        bots = await self.__fetcher.get(endpoints.GET_TOP_BOTS.format(value=sort_by.value))
        return list(map(parse_bot_object, bots))

    async def get_user_bots(self, id: Union[int, str]) -> UserBots:
        """
        Returns all bots from a user. (As a UserBots object)

        :param id: The user from which the bots should be fetched.
        """
        user_bots = await self.__fetcher.get(endpoints.GET_USER_BOTS.format(id=id))
        print(user_bots)
        return UserBots(parse_user_base_object(user_bots["user"]), list(map(parse_bot_object, user_bots["bots"])))



# -*- coding: utf-8 -*-
# MIT License
# Copyright (c) 2020 Arthur
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from datetime import datetime
from typing import Dict, Union, List

from ..objects import Bot, BotBase, UserBase


def parse_bot_object(data: Dict[str, Union[bool, List[str], str, int]]):
    return Bot(data["certified"], list(map(lambda i: int(i), data.get("owners"))), data.get("deleted"),
               data.get("id"),
               data.get("name"), data.get("def_avatar"), data.get("avatar"), data.get("short_desc"),
               data.get("lib"),
               data.get("prefix"), data.get("website"),
               datetime.strptime(data.get("approved_at"), "%Y-%m-%dT%H:%M:%S.%fZ"), data.get("monthly_votes"),
               data.get("server_count"), data.get("total_votes"), data.get("shard_count"),
               data.get("monthly_votes_rank"), data.get("server_count_rank"), data.get("total_votes_rank"),
               data.get("shard_count_rank"), datetime.strptime(data.get("timestamp"), "%Y-%m-%dT%H:%M:%S.%fZ"))


def parse_bot_base_object(data: Dict[str, Union[List[str], int, str]]) -> BotBase:
    return BotBase(list(map(lambda owner: int(owner), data.get("owners"))), data.get("monthly_votes"),
                   data.get("server_count"), data.get("total_votes"), data.get("shard_count"),
                   data.get("monthly_votes_rank"), data.get("server_count_rank"), data.get("total_votes_rank"),
                   data.get("shard_count_rank"),
                   datetime.strptime(data.get("timestamp"), "%Y-%m-%dT%H:%M:%S.%fZ"))


def parse_user_base_object(data: Dict[str, str]) -> UserBase:
    return UserBase(data.get("id"), data.get("tag"), data.get("avatar"), data.get("def_avatar"))



# -*- coding: utf-8 -*-
# MIT License
# Copyright (c) 2020 Arthur
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from datetime import datetime
from typing import List

from ..utils import represents


class BotBase:
    """A bot helper function that contains the basic bot values"""

    def __init__(self, owners: List[int], monthly_votes: int, server_count: int, total_votes: int,
                 shard_count: int, monthly_votes_rank: int, server_count_rank: int, total_votes_rank: int,
                 shard_count_rank: int, updated_at: datetime):
        """
        Creates a dblstats BotBase object, which contains the basic bot values.

        :param owners: A collection of the bot owner(s) their discord IDs
        :param monthly_votes: The amount of votes that the bot has received this month
        :param server_count: The amount of servers the bot is in (self reported)
        :param total_votes: The total amount of votes that the bot has
        :param shard_count: The amount of shards that the bot is running on (self reported)
        :param monthly_votes_rank: The position that the bot is in relative to the other bots registered on Top.gg qua votes from this month
        :param server_count_rank: The position that the bot is in relative to the other bots registered on Top.gg qua servers
        :param total_votes_rank: The position that the bot is in relative to the other bots registered on Top.gg qua votes
        :param shard_count_rank: The position that the bot is in relative to the other bots registered on Top.gg qua shards
        :param updated_at: The timestamp that this was updated on
        """
        self.owners = owners
        self.monthly_votes = monthly_votes
        self.server_count = server_count
        self.total_votes = total_votes
        self.shard_count = shard_count
        self.monthly_votes_rank = monthly_votes_rank
        self.server_count_rank = server_count_rank
        self.total_votes_rank = total_votes_rank
        self.shard_count_rank = shard_count_rank
        self.updated_at = updated_at

    def __repr__(self) -> str:
        return represents(self)


class Bot(BotBase):
    """The object that represents a bot in Top.gg"""

    def __init__(self, certified: bool, owners: List[int], deleted: bool, id: str, name: str, default_avatar: str,
                 avatar: str, short_description: str, library: str, prefix: str, website: str, approved_at: datetime,
                 monthly_votes: int, server_count: int, total_votes: int, shard_count: int, monthly_votes_rank: int,
                 server_count_rank: int, total_votes_rank: int, shard_count_rank: int, updated_at: datetime):
        """
        Create a dblstats Bot object. (which represents a bot in Top.gg)

        :param certified: Whether or not the bot is certified on Top.gg
        :param deleted: Whether or not the bot has been removed from Top.gg
        :param id: The bot its user id
        :param name: The bot its username
        :param default_avatar: The default assigned avatar that has been given to the bot (Decided by discriminator)
        :param avatar: The bot avatar url
        :param short_description: A short text that describes the bot
        :param library: The library that the bot uses
        :param prefix: The default bot prefix
        :param website: The website that is associated with the bot
        :param approved_at: The datetime from when the bot got added to Top.gg
        """
        super().__init__(owners, monthly_votes, server_count, total_votes, shard_count, monthly_votes_rank,
                         server_count_rank, total_votes_rank, shard_count_rank, updated_at)
        self.certified = certified
        self.deleted = deleted
        self.id = id
        self.name = name
        self.default_avatar = f"https://discord.com/assets/{default_avatar}.png"
        self.avatar = f"https://cdn.discordapp.com/avatars/{id}/{avatar}.webp"
        self.short_description = short_description
        self.library = library
        self.prefix = prefix
        self.website = website
        self.approved_at = approved_at

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return represents(self)


class BotHistory:
    """
    An object that contains a collection of `BotBase` objects.
    Each item differs one hour with the previous item.

    The maximum collection size is 500
    """

    def __init__(self, id: str, history: List[BotBase]):
        """
        Creates a dblstats BotHistory object.
        In this object you can see/find the previous bot data for that bot.

        :param id: The bot its id from which the history is shown
        :param history: A collection of `BotBase` objects that can be used to see the bot its history.
        """
        self.id = id
        self.history = history

    def __iter__(self) -> List[BotBase]:
        return self.history

    def __repr__(self) -> str:
        return represents(self)



# -*- coding: utf-8 -*-
# MIT License
# Copyright (c) 2020 Arthur
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from datetime import datetime
from typing import List, Dict, Union

from .user import UserBase
from ..utils import represents, endpoints, AsyncFetcher


class AuctionTag:
    """Represents a dblstats tag object."""

    def __init__(self, bot: List[str], server: List[str]):
        self.bot = bot
        self.server = server

    def __repr__(self):
        return represents(self)


class AuctionUser(UserBase):
    """Represents a Top.gg auction user."""

    def __init__(self, id: str, user_tag: str, avatar: str, default_avatar: str, product: str, tag: str, list: str,
                 amount: int, outbid: bool, timestamp: datetime, is_fake: bool):
        """Creates a Top.gg auction user."""
        super().__init__(id, user_tag, avatar, default_avatar)
        self.product = product
        self.tag = tag
        self.list = list
        self.amount = amount
        self.outbid = outbid
        self.timestamp = timestamp
        self.is_fake = is_fake

    def __repr__(self):
        return represents(self)


class AuctionCurrent:
    """Represents all current betters."""

    def __init__(self, bot: Dict[str, List[AuctionUser]], server: Dict[str, List[AuctionUser]]):
        """Creates a new auction current object."""
        self.bot = bot
        self.server = server

    def __repr__(self):
        return represents(self)


class Auctions:
    """An object that can fetch the current Top.gg auctions"""

    def __init__(self, fetcher: AsyncFetcher):
        self.__fetcher = fetcher

    def __repr__(self):
        return represents(self)

    async def get_tags(self) -> AuctionTag:
        """Returns a collection of bot and server tags."""
        tags = await self.__fetcher.get(endpoints.GET_AUCTIONS_TAGS)
        return AuctionTag(tags["bot"], tags["server"])

    @staticmethod
    def parse_user_base_obj(data: Dict[str, Union[Dict[str, str], str, bool]]) -> AuctionUser:
        return AuctionUser(data["user"].get("id"), data["user"].get("tag"), data["user"].get("avatar"),
                           data["user"].get("def_avatar"), data.get("product"), data.get("tag"),
                           data.get("list"), data.get("amount"), data.get("outbid"),
                           datetime.strptime(data.get("timestamp"), "%Y-%m-%dT%H:%M:%S.%fZ"),
                           data.get("is_fake"))

    def parse_auction_users(self, database: Dict[str, List[Dict[str, Union[bool, int, Dict[str, str]]]]]) -> \
            Dict[str, List[AuctionUser]]:
        data = {}
        for key, collection in database:
            data[key] = list(map(self.parse_user_base_obj, collection))
        return data

    async def get_current(self) -> AuctionCurrent:
        """Returns a dictionary of all current betters."""
        data = await self.__fetcher.get(endpoints.GET_AUCTIONS_CURRENT)
        return AuctionCurrent(self.parse_auction_users(data["bot"].items()),
                              self.parse_auction_users(data["server"].items()))



from distutils.core import setup
from os import path

from dblstats.version import version

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='dblstats',
    packages=['dblstats', 'dblstats.objects', 'dblstats.utils'],
    version=version,
    license='MIT',
    description='A python dblstats.com API wrapper.',
    # project_urls={
    #     "Documentation": "WIP",
    # },
    long_description=long_description,
    author='Arthurdw',
    author_email='mail.arthurdw@gmail.com',
    url='https://github.com/Arthurdw/dblstats',
    download_url=f'https://github.com/Arthurdw/dblstats/archive/{version}.tar.gz',
    keywords=["dbl", "stats", "statistics", "dblstatistics.com"],
    install_requires=[
        "aiohttp"
    ],
    classifiers=[
        # 'Development Status :: 2 - Pre-Alpha',
        # Development statuses:
        # Development Status :: 1 - Planning
        # Development Status :: 2 - Pre-Alpha
        # Development Status :: 3 - Alpha
        'Development Status :: 4 - Beta',
        # Development Status :: 5 - Production/Stable
        # Development Status :: 6 - Mature
        # Development Status :: 7 - Inactive
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
